import { isUndefined } from 'lodash';
import { useMemo } from 'react';
import { flushSync } from 'react-dom';
import { fromEvent, Subject, takeUntil } from 'rxjs';
import { useUnmount } from './useUnmount';
class BaseAsyncCapture {
    constructor() {
        Object.defineProperty(this, "tids", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "onDestroy$", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Subject()
        });
    }
    clearAll() {
        for (const cb of this.tids.values()) {
            cb();
        }
        this.onDestroy$.next();
        this.onDestroy$.complete();
    }
    fromEvent(...args) {
        return fromEvent(...args).pipe(takeUntil(this.onDestroy$));
    }
    setTimeout(handler, timeout) {
        const tid = window.setTimeout(handler, timeout);
        const clear = () => {
            clearTimeout(tid);
            this.tids.delete(tid);
        };
        this.tids.set(tid, clear);
        return clear;
    }
    requestAnimationFrame(...args) {
        const tid = requestAnimationFrame(...args);
        const clear = () => {
            cancelAnimationFrame(tid);
            this.tids.delete(tid);
        };
        this.tids.set(tid, clear);
        return clear;
    }
    onResize(el, fn) {
        let isFirst = true;
        const observer = new ResizeObserver(() => {
            if (isFirst) {
                isFirst = false;
            }
            else {
                flushSync(() => fn());
            }
        });
        observer.observe(el);
        const tid = Symbol();
        const clear = () => {
            observer.disconnect();
            this.tids.delete(tid);
        };
        this.tids.set(tid, clear);
        return clear;
    }
}
export class AsyncCapture extends BaseAsyncCapture {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "groups", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
    }
    createGroup(id) {
        let key = Symbol();
        const group = new BaseAsyncCapture();
        if (isUndefined(id)) {
            this.groups.set(key, group);
        }
        else {
            key = id;
            const oldGroup = this.groups.get(key);
            if (oldGroup) {
                oldGroup.clearAll();
            }
            this.groups.set(key, group);
        }
        return [group, key];
    }
    deleteGroup(id) {
        const group = this.groups.get(id);
        if (group) {
            group.clearAll();
            this.groups.delete(id);
        }
    }
    clearAll() {
        for (const cb of this.tids.values()) {
            cb();
        }
        this.onDestroy$.next();
        this.onDestroy$.complete();
        for (const group of this.groups.values()) {
            group.clearAll();
        }
    }
}
export function useAsync() {
    const asyncCapture = useMemo(() => new AsyncCapture(), []);
    useUnmount(() => {
        asyncCapture.clearAll();
    });
    return asyncCapture;
}
//# sourceMappingURL=useAsync.js.map